@@ -5,21 +5,17 @@
 # definitions of spatial distribution characters
 
 import math
-import statistics
-import warnings
-from distutils.version import LooseVersion
 
+import networkx as nx
 import numpy as np
 import pandas as pd
-import geopandas as gpd
 from tqdm import tqdm  # progress bar
 
 from .utils import _azimuth
 
-GPD_08 = str(gpd.__version__) >= LooseVersion("0.8.0")
-
 __all__ = [
     "Orientation",
+    "SharedWalls",
     "SharedWallsRatio",
     "StreetAlignment",
     "CellAlignment",
@@ -110,7 +106,55 @@ class Orientation:
         self.series = pd.Series(results_list, index=gdf.index)
 
 
-class SharedWallsRatio:
+class SharedWalls:
+    """
+    Calculate the length of shared walls of adjacent elements (typically buildings)
+
+    .. math::
+        \\textit{length of shared walls}
+
+    Note that data needs to be topologically correct. Overlapping polygons will lead to
+    incorrect results.
+
+    Adapted from :cite:`hamaina2012a`.
+
+    Parameters
+    ----------
+    gdf : GeoDataFrame
+        GeoDataFrame containing gdf to analyse
+
+    Attributes
+    ----------
+    series : Series
+        Series containing resulting values
+    gdf : GeoDataFrame
+        original GeoDataFrame
+
+    Examples
+    --------
+    >>> buildings_df['swr'] = momepy.SharedWalls(buildings_df).series
+
+    See also
+    --------
+    SharedWallsRatio
+    """
+
+    def __init__(self, gdf):
+        self.gdf = gdf
+
+        inp, res = gdf.sindex.query_bulk(gdf.geometry, predicate="intersects")
+        left = gdf.geometry.take(inp).reset_index(drop=True)
+        right = gdf.geometry.take(res).reset_index(drop=True)
+        intersections = left.intersection(right).length
+        results = intersections.groupby(inp).sum().reset_index(
+            drop=True
+        ) - gdf.geometry.length.reset_index(drop=True)
+        results.index = gdf.index
+
+        self.series = results
+
+
+class SharedWallsRatio(SharedWalls):
     """
     Calculate shared walls ratio of adjacent elements (typically buildings)
 
@@ -126,8 +170,7 @@ class SharedWallsRatio:
     ----------
     gdf : GeoDataFrame
         GeoDataFrame containing gdf to analyse
-    unique_id : (deprecated)
-    perimeters : str, list, np.array, pd.Series (default None)
+    perimeters : str, list, np.array, pd.Series (default None, optional)
         the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored perimeter value
 
     Attributes
@@ -144,19 +187,14 @@ class SharedWallsRatio:
     >>> buildings_df['swr'] = momepy.SharedWallsRatio(buildings_df).series
     >>> buildings_df['swr'][10]
     0.3424804411228673
+
+    See also
+    --------
+    SharedWalls
     """
 
-    def __init__(self, gdf, unique_id=None, perimeters=None):
-        if not GPD_08:
-            raise ImportError(
-                "The 'geopandas' >= 0.8.0 package is required to use SharedWallsRatio."
-            )
-        if unique_id is not None:
-            warnings.warn(
-                "unique_id is deprecated and will be removed in v0.4.", FutureWarning,
-            )
-
-        self.gdf = gdf
+    def __init__(self, gdf, perimeters=None):
+        super(SharedWallsRatio, self).__init__(gdf)
 
         if perimeters is None:
             self.perimeters = gdf.geometry.length
@@ -165,17 +203,7 @@ class SharedWallsRatio:
         else:
             self.perimeters = perimeters
 
-        inp, res = gdf.sindex.query_bulk(gdf.geometry, predicate="intersects")
-        left = gdf.geometry.take(inp).reset_index(drop=True)
-        right = gdf.geometry.take(res).reset_index(drop=True)
-        intersections = left.intersection(right).length
-        results = (
-            intersections.groupby(inp).sum().reset_index(drop=True)
-            - self.perimeters.reset_index(drop=True)
-        ) / self.perimeters.reset_index(drop=True)
-        results.index = gdf.index
-
-        self.series = results
+        self.series = self.series / self.perimeters
 
 
 class StreetAlignment:
@@ -434,12 +462,10 @@ class Alignment:
             data.iteritems(), total=data.shape[0], disable=not verbose
         ):
             if index in spatial_weights.neighbors.keys():
-                neighbours = spatial_weights.neighbors[index].copy()
+                neighbours = spatial_weights.neighbors[index]
                 if neighbours:
                     orientation = data.loc[neighbours]
-                    deviations = abs(orientation - orient)
-
-                    results_list.append(statistics.mean(deviations))
+                    results_list.append(abs(orientation - orient).mean())
                 else:
                     results_list.append(np.nan)
             else:
@@ -502,7 +528,7 @@ class NeighborDistance:
         for index, geom in tqdm(
             data.iteritems(), total=data.shape[0], disable=not verbose
         ):
-            if index in spatial_weights.neighbors.keys():
+            if geom is not None and index in spatial_weights.neighbors.keys():
                 neighbours = spatial_weights.neighbors[index]
                 building_neighbours = data.loc[neighbours]
                 if len(building_neighbours) > 0:
@@ -522,7 +548,7 @@ class MeanInterbuildingDistance:
     Calculate the mean interbuilding distance
 
     Interbuilding distances are calculated between buildings on adjacent cells based on
-    ``spatial_weights``, while the extent is defined in ``spatial_weights_higher``.
+    ``spatial_weights``, while the extent is defined as order of contiguity.
 
     .. math::
 
@@ -535,11 +561,8 @@ class MeanInterbuildingDistance:
         name of the column with unique id used as ``spatial_weights`` index
     spatial_weights : libpysal.weights
         spatial weights matrix
-    spatial_weights_higher : libpysal.weights, optional
-        spatial weights matrix - If None, Queen contiguity of a higher ``order`` will be calculated
-        based on ``spatial_weights``
     order : int
-        Order of Queen contiguity
+        Order of contiguity defining the extent
     verbose : bool (default True)
         if True, shows progress bars in loops and indication of steps
 
@@ -556,7 +579,7 @@ class MeanInterbuildingDistance:
     sw_higher : libpysal.weights
         Spatial weights matrix of higher order
     order : int
-        Order of Queen contiguity (only if spatial_weights_higher was not set)
+        Order of contiguity.
 
     Notes
     -----
@@ -565,7 +588,6 @@ class MeanInterbuildingDistance:
     Examples
     --------
     >>> buildings_df['mean_interbuilding_distance'] = momepy.MeanInterbuildingDistance(buildings_df, sw, 'uID').series
-    Generating weights matrix (Queen) of 3 topological steps...
     Computing mean interbuilding distances...
     100%|██████████| 144/144 [00:00<00:00, 317.42it/s]
     >>> buildings_df['mean_interbuilding_distance'][0]
@@ -585,17 +607,6 @@ class MeanInterbuildingDistance:
         self.sw = spatial_weights
         self.id = gdf[unique_id]
 
-        if spatial_weights_higher is None:
-            print(
-                f"Generating weights matrix (Queen) of {order} topological steps..."
-            ) if verbose else None
-            self.order = order
-            from momepy import sw_high
-
-            # matrix to define area of analysis (more steps)
-            spatial_weights_higher = sw_high(k=order, weights=spatial_weights)
-        self.sw_higher = spatial_weights_higher
-
         data = gdf.set_index(unique_id).geometry
 
         # define empty list for results
@@ -603,27 +614,28 @@ class MeanInterbuildingDistance:
 
         # define adjacency list from lipysal
         adj_list = spatial_weights.to_adjlist()
-        adj_list["distance"] = (
+        adj_list["weight"] = (
             data.loc[adj_list.focal]
-            .reset_index()
-            .distance(data.loc[adj_list.neighbor].reset_index())
+            .reset_index(drop=True)
+            .distance(data.loc[adj_list.neighbor].reset_index(drop=True))
+            .values
         )
 
-        print("Computing mean interbuilding distances...")
-        # iterate over objects to get the final values
-        for uid in tqdm(data.index, total=data.shape[0], disable=not verbose):
-            # define neighbours based on weights matrix defining analysis area
-            if uid in spatial_weights_higher.neighbors.keys():
-                neighbours = spatial_weights_higher.neighbors[uid].copy()
-                neighbours.append(uid)
-                if neighbours:
-                    selection = adj_list[adj_list.focal.isin(neighbours)][
-                        adj_list.neighbor.isin(neighbours)
-                    ]
-                    results_list.append(np.nanmean(selection.distance))
-            else:
-                results_list.append(np.nan)
+        # generate graph
+        G = nx.from_pandas_edgelist(
+            adj_list, source="focal", target="neighbor", edge_attr="weight"
+        )
 
+        print("Computing mean interbuilding distances...") if verbose else None
+        # iterate over subgraphs to get the final values
+        for uid in tqdm(data.index, total=data.shape[0], disable=not verbose):
+            try:
+                sub = nx.ego_graph(G, uid, radius=order)
+                results_list.append(
+                    np.nanmean([x[-1] for x in list(sub.edges.data("weight"))])
+                )
+            except Exception:
+                results_list.append(np.nan)
         self.series = pd.Series(results_list, index=gdf.index)
 
 
@@ -659,11 +671,6 @@ class NeighboringStreetOrientationDeviation:
     """
 
     def __init__(self, gdf):
-        if not GPD_08:
-            raise ImportError(
-                "The 'geopandas' >= 0.8.0 package is required to use "
-                "NeighboringStreetOrientationDeviation."
-            )
         self.gdf = gdf
         self.orientation = gdf.geometry.apply(self._orient)
 
