@@ -18,6 +18,7 @@ __all__ = [
     "Unique",
     "simpson_diversity",
     "shannon_diversity",
+    "Percentiles",
 ]
 
 
@@ -101,11 +102,8 @@ class Range:
         results_list = []
         for index in tqdm(data.index, total=data.shape[0], disable=not verbose):
             if index in spatial_weights.neighbors.keys():
-                neighbours = spatial_weights.neighbors[index].copy()
-                if neighbours:
-                    neighbours.append(index)
-                else:
-                    neighbours = [index]
+                neighbours = [index]
+                neighbours += spatial_weights.neighbors[index]
 
                 values_list = data.loc[neighbours]
                 results_list.append(sp.stats.iqr(values_list, rng=rng, **kwargs))
@@ -189,11 +187,8 @@ class Theil:
         results_list = []
         for index in tqdm(data.index, total=data.shape[0], disable=not verbose):
             if index in spatial_weights.neighbors.keys():
-                neighbours = spatial_weights.neighbors[index].copy()
-                if neighbours:
-                    neighbours.append(index)
-                else:
-                    neighbours = [index]
+                neighbours = [index]
+                neighbours += spatial_weights.neighbors[index]
 
                 values_list = data.loc[neighbours]
 
@@ -292,19 +287,9 @@ class Simpson:
     ):
         if not categorical:
             try:
-                import mapclassify.classifiers as classifiers
+                from mapclassify import classify
             except ImportError:
-                raise ImportError("The 'mapclassify' package is required")
-
-            schemes = {}
-            for classifier in classifiers.CLASSIFIERS:
-                schemes[classifier.lower()] = getattr(classifiers, classifier)
-            binning = binning.lower()
-            if binning not in schemes:
-                raise ValueError(
-                    "Invalid binning. Binning must be in the"
-                    " set: %r" % schemes.keys()
-                )
+                raise ImportError("The 'mapclassify >= 2.4.2` package is required.")
 
         self.gdf = gdf
         self.sw = spatial_weights
@@ -329,18 +314,15 @@ class Simpson:
             categories = data.unique()
 
         if not categorical:
-            self.bins = schemes[binning](data, **classification_kwds).bins
+            self.bins = classify(data, scheme=binning, **classification_kwds).bins
         else:
             self.bins = categories
 
         results_list = []
         for index in tqdm(data.index, total=data.shape[0], disable=not verbose):
             if index in spatial_weights.neighbors.keys():
-                neighbours = spatial_weights.neighbors[index].copy()
-                if neighbours:
-                    neighbours.append(index)
-                else:
-                    neighbours = [index]
+                neighbours = [index]
+                neighbours += spatial_weights.neighbors[index]
                 values_list = data.loc[neighbours]
 
                 results_list.append(
@@ -372,7 +354,7 @@ def simpson_diversity(data, bins=None, categorical=False, categories=None):
         \\lambda=\\sum_{i=1}^{R} p_{i}^{2}
 
     Formula adapted from https://gist.github.com/martinjc/f227b447791df8c90568.
-    
+
     Parameters
     ----------
     data : GeoDataFrame
@@ -388,7 +370,7 @@ def simpson_diversity(data, bins=None, categorical=False, categories=None):
     -------
     float
         Simpson's diversity index
-    
+
     See also
     --------
     momepy.Simpson : Calculates the Simpson\'s diversity index of values within neighbours
@@ -588,19 +570,9 @@ class Shannon:
     ):
         if not categorical:
             try:
-                import mapclassify.classifiers as classifiers
+                from mapclassify import classify
             except ImportError:
-                raise ImportError("The 'mapclassify' package is required")
-
-            schemes = {}
-            for classifier in classifiers.CLASSIFIERS:
-                schemes[classifier.lower()] = getattr(classifiers, classifier)
-            binning = binning.lower()
-            if binning not in schemes:
-                raise ValueError(
-                    "Invalid binning. Binning must be in the"
-                    " set: %r" % schemes.keys()
-                )
+                raise ImportError("The 'mapclassify >= 2.4.2` package is required.")
 
         self.gdf = gdf
         self.sw = spatial_weights
@@ -623,18 +595,15 @@ class Shannon:
             categories = data.unique()
 
         if not categorical:
-            self.bins = schemes[binning](data, **classification_kwds).bins
+            self.bins = classify(data, scheme=binning, **classification_kwds).bins
         else:
             self.bins = categories
 
         results_list = []
         for index in tqdm(data.index, total=data.shape[0], disable=not verbose):
             if index in spatial_weights.neighbors.keys():
-                neighbours = spatial_weights.neighbors[index].copy()
-                if neighbours:
-                    neighbours.append(index)
-                else:
-                    neighbours = [index]
+                neighbours = [index]
+                neighbours += spatial_weights.neighbors[index]
                 values_list = data.loc[neighbours]
 
                 results_list.append(
@@ -661,7 +630,7 @@ def shannon_diversity(data, bins=None, categorical=False, categories=None):
         \\lambda=\\sum_{i=1}^{R} p_{i}^{2}
 
     Formula adapted from https://gist.github.com/audy/783125
-    
+
     Parameters
     ----------
     data : GeoDataFrame
@@ -677,7 +646,7 @@ def shannon_diversity(data, bins=None, categorical=False, categories=None):
     -------
     float
         Shannon's diversity index
-    
+
     See also
     --------
     momepy.Shannon : Calculates the Shannon's diversity index of values within neighbours
@@ -769,11 +738,8 @@ class Unique:
         results_list = []
         for index in tqdm(data.index, total=data.shape[0], disable=not verbose):
             if index in spatial_weights.neighbors.keys():
-                neighbours = spatial_weights.neighbors[index].copy()
-                if neighbours:
-                    neighbours.append(index)
-                else:
-                    neighbours = [index]
+                neighbours = [index]
+                neighbours += spatial_weights.neighbors[index]
 
                 values_list = data.loc[neighbours]
                 results_list.append(len(values_list.unique()))
@@ -781,3 +747,97 @@ class Unique:
                 results_list.append(np.nan)
 
         self.series = pd.Series(results_list, index=gdf.index)
+
+
+class Percentiles:
+    """
+    Calculates the percentiles of values within neighbours defined in ``spatial_weights``.
+
+    Parameters
+    ----------
+    gdf : GeoDataFrame
+        GeoDataFrame containing morphological tessellation
+    values : str, list, np.array, pd.Series
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored character value.
+    spatial_weights : libpysal.weights
+        spatial weights matrix
+    unique_id : str
+        name of the column with unique id used as ``spatial_weights`` index
+    percentiles : array-like (default [25, 50, 75])
+        percentiles to return
+    interpolation : {'linear', 'lower', 'higher', 'midpoint', 'nearest'}
+        This optional parameter specifies the interpolation method to
+        use when the desired percentile lies between two data points
+        ``i < j``:
+
+        * ``'linear'``
+        * ``'lower'``
+        * ``'higher'``
+        * ``'nearest'``
+        * ``'midpoint'``
+
+        See the documentation of ``numpy.percentile`` for details.
+   
+    verbose : bool (default True)
+        if True, shows progress bars in loops and indication of steps
+
+    Attributes
+    ----------
+    frame : DataFrame
+        DataFrame containing resulting values
+    gdf : GeoDataFrame
+        original GeoDataFrame
+    values : Series
+        Series containing used values
+    sw : libpysal.weights
+        spatial weights matrix
+    id : Series
+        Series containing used unique ID
+
+    Examples
+    --------
+    >>> sw = momepy.sw_high(k=3, gdf=tessellation_df, ids='uID')
+    >>> tessellation_df['cluster_unique'] = mm.Percentiles(tessellation_df, 'cluster', sw, 'uID').frame
+    100%|██████████| 144/144 [00:00<00:00, 722.50it/s]
+    """
+
+    def __init__(
+        self,
+        gdf,
+        values,
+        spatial_weights,
+        unique_id,
+        percentiles=[25, 50, 75],
+        interpolation="midpoint",
+        verbose=True,
+    ):
+        self.gdf = gdf
+        self.sw = spatial_weights
+        self.id = gdf[unique_id]
+
+        data = gdf.copy()
+
+        if values is not None:
+            if not isinstance(values, str):
+                data["mm_v"] = values
+                values = "mm_v"
+        self.values = data[values]
+
+        data = data.set_index(unique_id)[values]
+
+        results_list = []
+        for index in tqdm(data.index, total=data.shape[0], disable=not verbose):
+            if index in spatial_weights.neighbors.keys():
+                neighbours = [index]
+                neighbours += spatial_weights.neighbors[index]
+
+                values_list = data.loc[neighbours]
+                results_list.append(
+                    np.nanpercentile(
+                        values_list, percentiles, interpolation=interpolation
+                    )
+                )
+            else:
+                results_list.append(np.nan)
+
+        self.frame = pd.DataFrame(results_list, columns=percentiles, index=gdf.index)
