@@ -393,6 +393,13 @@ def _circle_area(points):
     return math.pi * circ[2] ** 2
 
 
+def _circle_radius(points):
+    if len(points[0]) == 3:
+        points = [x[:2] for x in points]
+    circ = _make_circle(points)
+    return circ[2]
+
+
 class CircularCompactness:
     """
     Calculates compactness index of each object in given GeoDataFrame.
@@ -407,7 +414,8 @@ class CircularCompactness:
     gdf : GeoDataFrame
         GeoDataFrame containing objects
     areas : str, list, np.array, pd.Series (default None)
-        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is stored area value. If set to ``None``, function will calculate areas
+        the name of the dataframe column, ``np.array``, or ``pd.Series`` where is
+        stored area value. If set to ``None``, function will calculate areas
         during the process without saving them separately.
 
     Attributes
@@ -421,26 +429,24 @@ class CircularCompactness:
 
     Examples
     --------
-    >>> buildings_df['circ_comp'] = momepy.CircularCompactness(buildings_df, 'area').series
-    >>> buildings_df['circ_comp'][0]
+    >>> buildings_df['comp'] = momepy.CircularCompactness(buildings_df, 'area').series
+    >>> buildings_df['comp'][0]
     0.572145421828038
     """
 
     def __init__(self, gdf, areas=None):
         self.gdf = gdf
 
-        gdf = gdf.copy()
-
         if areas is None:
             areas = gdf.geometry.area
-        if not isinstance(areas, str):
-            gdf["mm_a"] = areas
-            areas = "mm_a"
-        self.areas = gdf[areas]
-        gdf["hull"] = gdf.convex_hull.exterior
-        self.series = gdf[[areas, "hull"]].apply(
-            lambda row: (row[areas]) / (_circle_area(list(row.hull.coords))), axis=1
+        elif isinstance(areas, str):
+            areas = gdf[areas]
+        self.areas = areas
+        hull = gdf.convex_hull.exterior
+        radius = hull.apply(
+            lambda g: _circle_radius(list(g.coords)) if g is not None else None
         )
+        self.series = areas / (np.pi * radius ** 2)
 
 
 class SquareCompactness:
@@ -721,7 +727,8 @@ class Corners:
     """
     Calculates number of corners of each object in given GeoDataFrame.
 
-    Uses only external shape (``shapely.geometry.exterior``), courtyards are not included.
+    Uses only external shape (``shapely.geometry.exterior``), courtyards are not
+    included.
 
     .. math::
         \\sum corner
@@ -765,6 +772,7 @@ class Corners:
             cosine_angle = np.dot(ba, bc) / (np.linalg.norm(ba) * np.linalg.norm(bc))
             angle = np.arccos(cosine_angle)
 
+            # TODO: add arg to specify these values
             if np.degrees(angle) <= 170:
                 return True
             if np.degrees(angle) >= 190:
@@ -773,33 +781,65 @@ class Corners:
 
         # fill new column with the value of area, iterating over rows one by one
         for geom in tqdm(gdf.geometry, total=gdf.shape[0], disable=not verbose):
-            corners = 0  # define empty variables
-            points = list(geom.exterior.coords)  # get points of a shape
-            stop = len(points) - 1  # define where to stop
-            for i in np.arange(
-                len(points)
-            ):  # for every point, calculate angle and add 1 if True angle
-                if i == 0:
-                    continue
-                elif i == stop:
-                    a = np.asarray(points[i - 1])
-                    b = np.asarray(points[i])
-                    c = np.asarray(points[1])
-
-                    if _true_angle(a, b, c) is True:
-                        corners = corners + 1
-                    else:
+            if geom.type == "Polygon":
+                corners = 0  # define empty variables
+                points = list(geom.exterior.coords)  # get points of a shape
+                stop = len(points) - 1  # define where to stop
+                for i in np.arange(
+                    len(points)
+                ):  # for every point, calculate angle and add 1 if True angle
+                    if i == 0:
                         continue
+                    elif i == stop:
+                        a = np.asarray(points[i - 1])
+                        b = np.asarray(points[i])
+                        c = np.asarray(points[1])
 
-                else:
-                    a = np.asarray(points[i - 1])
-                    b = np.asarray(points[i])
-                    c = np.asarray(points[i + 1])
+                        if _true_angle(a, b, c) is True:
+                            corners = corners + 1
+                        else:
+                            continue
 
-                    if _true_angle(a, b, c) is True:
-                        corners = corners + 1
                     else:
-                        continue
+                        a = np.asarray(points[i - 1])
+                        b = np.asarray(points[i])
+                        c = np.asarray(points[i + 1])
+
+                        if _true_angle(a, b, c) is True:
+                            corners = corners + 1
+                        else:
+                            continue
+            elif geom.type == "MultiPolygon":
+                corners = 0  # define empty variables
+                for g in geom.geoms:
+                    points = list(g.exterior.coords)  # get points of a shape
+                    stop = len(points) - 1  # define where to stop
+                    for i in np.arange(
+                        len(points)
+                    ):  # for every point, calculate angle and add 1 if True angle
+                        if i == 0:
+                            continue
+                        elif i == stop:
+                            a = np.asarray(points[i - 1])
+                            b = np.asarray(points[i])
+                            c = np.asarray(points[1])
+
+                            if _true_angle(a, b, c) is True:
+                                corners = corners + 1
+                            else:
+                                continue
+
+                        else:
+                            a = np.asarray(points[i - 1])
+                            b = np.asarray(points[i])
+                            c = np.asarray(points[i + 1])
+
+                            if _true_angle(a, b, c) is True:
+                                corners = corners + 1
+                            else:
+                                continue
+            else:
+                corners = np.nan
 
             results_list.append(corners)
 
@@ -819,6 +859,8 @@ class Squareness:
 
     Adapted from :cite:`dibble2017`.
 
+    Returns ``np.nan`` for MultiPolygons.
+
     Parameters
     ----------
     gdf : GeoDataFrame
@@ -941,27 +983,25 @@ class EquivalentRectangularIndex:
     def __init__(self, gdf, areas=None, perimeters=None):
         self.gdf = gdf
         # define empty list for results
-        gdf = gdf.copy()
 
         if perimeters is None:
-            gdf["mm_p"] = gdf.geometry.length
-            perimeters = "mm_p"
+            perimeters = gdf.geometry.length
         else:
-            if not isinstance(perimeters, str):
-                gdf["mm_p"] = perimeters
-                perimeters = "mm_p"
-        self.perimeters = gdf[perimeters]
+            if isinstance(perimeters, str):
+                perimeters = gdf[perimeters]
+
+        self.perimeters = perimeters
+
         if areas is None:
-            gdf["mm_a"] = gdf.geometry.area
-            areas = "mm_a"
+            areas = gdf.geometry.area
         else:
-            if not isinstance(areas, str):
-                gdf["mm_a"] = areas
-                areas = "mm_a"
-        self.areas = gdf[areas]
+            if isinstance(areas, str):
+                areas = gdf[areas]
+
+        self.areas = areas
         # TODO: vectorize minimum_rotated_rectangle after pygeos implementation
         bbox = gdf.geometry.apply(lambda g: g.minimum_rotated_rectangle)
-        res = np.sqrt(gdf[areas] / bbox.area) * (bbox.length / gdf[perimeters])
+        res = np.sqrt(areas / bbox.area) * (bbox.length / perimeters)
 
         self.series = pd.Series(res, index=gdf.index)
 
@@ -1031,6 +1071,8 @@ class CentroidCorners:
 
     Adapted from :cite:`schirmer2015` and :cite:`cimburova2017`.
 
+    Returns ``np.nan`` for MultiPolygons.
+
     Parameters
     ----------
     gdf : GeoDataFrame
@@ -1081,54 +1123,57 @@ class CentroidCorners:
 
         # iterating over rows one by one
         for geom in tqdm(gdf.geometry, total=gdf.shape[0], disable=not verbose):
-            distances = []  # set empty list of distances
-            centroid = geom.centroid  # define centroid
-            points = list(geom.exterior.coords)  # get points of a shape
-            stop = len(points) - 1  # define where to stop
-            for i in np.arange(
-                len(points)
-            ):  # for every point, calculate angle and add 1 if True angle
-                if i == 0:
-                    continue
-                elif i == stop:
-                    a = np.asarray(points[i - 1])
-                    b = np.asarray(points[i])
-                    c = np.asarray(points[1])
-                    p = Point(points[i])
-
-                    if true_angle(a, b, c) is True:
-                        distance = centroid.distance(
-                            p
-                        )  # calculate distance point - centroid
-                        distances.append(distance)  # add distance to the list
-                    else:
+            if geom.type == "Polygon":
+                distances = []  # set empty list of distances
+                centroid = geom.centroid  # define centroid
+                points = list(geom.exterior.coords)  # get points of a shape
+                stop = len(points) - 1  # define where to stop
+                for i in np.arange(
+                    len(points)
+                ):  # for every point, calculate angle and add 1 if True angle
+                    if i == 0:
                         continue
+                    elif i == stop:
+                        a = np.asarray(points[i - 1])
+                        b = np.asarray(points[i])
+                        c = np.asarray(points[1])
+                        p = Point(points[i])
 
-                else:
-                    a = np.asarray(points[i - 1])
-                    b = np.asarray(points[i])
-                    c = np.asarray(points[i + 1])
-                    p = Point(points[i])
+                        if true_angle(a, b, c) is True:
+                            distance = centroid.distance(
+                                p
+                            )  # calculate distance point - centroid
+                            distances.append(distance)  # add distance to the list
+                        else:
+                            continue
 
-                    if true_angle(a, b, c) is True:
-                        distance = centroid.distance(p)
-                        distances.append(distance)
                     else:
-                        continue
-            if not distances:  # circular buildings
-                from momepy.dimension import _longest_axis
+                        a = np.asarray(points[i - 1])
+                        b = np.asarray(points[i])
+                        c = np.asarray(points[i + 1])
+                        p = Point(points[i])
 
-                if geom.has_z:
-                    coords = [
-                        (coo[0], coo[1]) for coo in geom.convex_hull.exterior.coords
-                    ]
+                        if true_angle(a, b, c) is True:
+                            distance = centroid.distance(p)
+                            distances.append(distance)
+                        else:
+                            continue
+                if not distances:  # circular buildings
+                    if geom.has_z:
+                        coords = [
+                            (coo[0], coo[1]) for coo in geom.convex_hull.exterior.coords
+                        ]
+                    else:
+                        coords = geom.convex_hull.exterior.coords
+                    results_list.append(_circle_radius(coords))
+                    results_list_sd.append(0)
                 else:
-                    coords = geom.convex_hull.exterior.coords
-                results_list.append(_longest_axis(coords) / 2)
-                results_list_sd.append(0)
+                    results_list.append(np.mean(distances))  # calculate mean
+                    results_list_sd.append(np.std(distances))  # calculate st.dev
             else:
-                results_list.append(np.mean(distances))  # calculate mean
-                results_list_sd.append(np.std(distances))  # calculate st.dev
+                results_list.append(np.nan)
+                results_list_sd.append(np.nan)
+
         self.mean = pd.Series(results_list, index=gdf.index)
         self.std = pd.Series(results_list_sd, index=gdf.index)
 
@@ -1140,7 +1185,7 @@ class Linearity:
     .. math::
         \\frac{l_{euclidean}}{l_{segment}}
 
-    where `l` is the length of the LineString.
+    where `l` is the length of the LineString. MultiLineString returns ``np.nan``.
 
     Adapted from :cite:`araldi2019`.
 
@@ -1170,6 +1215,8 @@ class Linearity:
 
         euclidean = gdf.geometry.apply(
             lambda geom: self._dist(geom.coords[0], geom.coords[-1])
+            if geom.type == "LineString"
+            else np.nan
         )
         self.series = euclidean / gdf.geometry.length
 
